// src/workers/prod/worker-facade.ts
import {
  ProdChildToWorkerMessages,
  ProdWorkerToChildMessages,
  clock,
  taskCatalog
} from "@trigger.dev/core/v3";
import {
  TaskExecutor,
  DurableClock,
  getEnvVar,
  logLevels,
  OtelTaskLogger,
  ConsoleInterceptor
} from "@trigger.dev/core/v3/workers";
import { ZodIpcConnection } from "@trigger.dev/core/v3/zodIpc";
import { ZodSchemaParsedError } from "@trigger.dev/core/v3/zodMessageHandler";
import "source-map-support/register.js";
import {
  TaskRunErrorCodes,
  TriggerTracer,
  logger,
  runtime
} from "@trigger.dev/core/v3";
import { ProdRuntimeManager } from "@trigger.dev/core/v3/prod";

// package.json
var version = "3.0.0-beta.31";

// src/workers/prod/worker-facade.ts
__WORKER_SETUP__;
__IMPORTED_PROJECT_CONFIG__;
var otelTracer = tracingSDK.getTracer("trigger-prod-worker", version);
var otelLogger = tracingSDK.getLogger("trigger-prod-worker", version);
var durableClock = new DurableClock();
clock.setGlobalClock(durableClock);
var tracer = new TriggerTracer({ tracer: otelTracer, logger: otelLogger });
var consoleInterceptor = new ConsoleInterceptor(otelLogger, true);
var triggerLogLevel = getEnvVar("TRIGGER_LOG_LEVEL");
var configLogLevel = triggerLogLevel ? triggerLogLevel : importedConfig ? importedConfig.logLevel : __PROJECT_CONFIG__.logLevel;
var otelTaskLogger = new OtelTaskLogger({
  logger: otelLogger,
  tracer,
  level: logLevels.includes(configLogLevel) ? configLogLevel : "info"
});
logger.setGlobalTaskLogger(otelTaskLogger);
var TaskFileImports = {};
var TaskFiles = {};
__TASKS__;
(() => {
  for (const [importName, taskFile] of Object.entries(TaskFiles)) {
    const fileImports = TaskFileImports[importName];
    for (const [exportName, task] of Object.entries(fileImports ?? {})) {
      if (typeof task === "object" && task !== null && "id" in task && typeof task.id === "string") {
        if (taskCatalog.taskExists(task.id)) {
          taskCatalog.registerTaskFileMetadata(task.id, {
            exportName,
            filePath: taskFile.filePath
          });
        }
      }
    }
  }
})();
var _execution;
var _isRunning = false;
var zodIpc = new ZodIpcConnection({
  listenSchema: ProdWorkerToChildMessages,
  emitSchema: ProdChildToWorkerMessages,
  process,
  handlers: {
    EXECUTE_TASK_RUN: async ({ execution, traceContext, metadata }, sender) => {
      if (_isRunning) {
        console.error("Worker is already running a task");
        await sender.send("TASK_RUN_COMPLETED", {
          execution,
          result: {
            ok: false,
            id: execution.run.id,
            error: {
              type: "INTERNAL_ERROR",
              code: TaskRunErrorCodes.TASK_ALREADY_RUNNING
            }
          }
        });
        return;
      }
      process.title = `trigger-prod-worker: ${execution.task.id} ${execution.run.id}`;
      const task = taskCatalog.getTask(execution.task.id);
      if (!task) {
        console.error(`Could not find task ${execution.task.id}`);
        await sender.send("TASK_RUN_COMPLETED", {
          execution,
          result: {
            ok: false,
            id: execution.run.id,
            error: {
              type: "INTERNAL_ERROR",
              code: TaskRunErrorCodes.COULD_NOT_FIND_EXECUTOR
            }
          }
        });
        return;
      }
      const executor = new TaskExecutor(task, {
        tracer,
        tracingSDK,
        consoleInterceptor,
        projectConfig: __PROJECT_CONFIG__,
        importedConfig,
        handleErrorFn: handleError
      });
      try {
        _execution = execution;
        _isRunning = true;
        const result = await executor.execute(execution, metadata, traceContext);
        return sender.send("TASK_RUN_COMPLETED", {
          execution,
          result
        });
      } finally {
        _execution = void 0;
        _isRunning = false;
      }
    },
    TASK_RUN_COMPLETED_NOTIFICATION: async ({ completion, execution }) => {
      prodRuntimeManager.resumeTask(completion, execution);
    },
    WAIT_COMPLETED_NOTIFICATION: async () => {
      prodRuntimeManager.resumeAfterDuration();
    },
    CLEANUP: async ({ flush, kill }, sender) => {
      if (kill) {
        await tracingSDK.flush();
        if (_execution) {
          await sender.send("TASK_RUN_COMPLETED", {
            execution: _execution,
            result: {
              ok: false,
              id: _execution.run.id,
              error: {
                type: "INTERNAL_ERROR",
                code: TaskRunErrorCodes.GRACEFUL_EXIT_TIMEOUT,
                message: "Worker process killed while attempt in progress."
              }
            }
          });
        }
        await sender.send("READY_TO_DISPOSE", void 0);
      } else {
        if (flush) {
          await tracingSDK.flush();
        }
      }
    }
  }
});
process.on("SIGTERM", async () => {
});
var prodRuntimeManager = new ProdRuntimeManager(zodIpc, {
  waitThresholdInMs: parseInt(process.env.TRIGGER_RUNTIME_WAIT_THRESHOLD_IN_MS ?? "30000", 10)
});
runtime.setGlobalRuntimeManager(prodRuntimeManager);
var TASK_METADATA = taskCatalog.getAllTaskMetadata();
zodIpc.send("TASKS_READY", { tasks: TASK_METADATA }).catch((err) => {
  if (err instanceof ZodSchemaParsedError) {
    zodIpc.send("TASKS_FAILED_TO_PARSE", { zodIssues: err.error.issues, tasks: TASK_METADATA });
  } else {
    console.error("Failed to send TASKS_READY message", err);
  }
});
process.title = "trigger-prod-worker";
async function asyncHeartbeat(initialDelayInSeconds = 30, intervalInSeconds = 5) {
  async function _doHeartbeat() {
    while (true) {
      if (_isRunning && _execution) {
        try {
          await zodIpc.send("TASK_HEARTBEAT", { id: _execution.attempt.id });
        } catch (err) {
          console.error("Failed to send HEARTBEAT message", err);
        }
      }
      await new Promise((resolve) => setTimeout(resolve, 1e3 * intervalInSeconds));
    }
  }
  await new Promise((resolve) => setTimeout(resolve, 1e3 * initialDelayInSeconds));
  return _doHeartbeat();
}
asyncHeartbeat(5).catch((err) => {
  console.error("Failed to start asyncHeartbeat", err);
});
